@@ -25,6 +25,7 @@ from barman import output
 from barman.infofile import BackupInfo
 from barman import lockfile
 from barman.server import Server
+import barman.diagnose
 import barman.config
 import logging
 import os
@@ -48,23 +49,23 @@ def list_server(minimal=False):
     output.close_and_exit()
 
 
-def cron(verbose=True):
+def cron():
     """
     Run maintenance tasks
     """
-    filename = os.path.join(barman.__config__.barman_home, '.cron.lock')
+    lockname = os.path.join(barman.__config__.barman_home, '.cron.lock')
     try:
-        with lockfile.LockFile(filename, raise_if_fail=True):
+        with lockfile.LockFile(lockname, raise_if_fail=True):
             servers = [Server(conf) for conf in barman.__config__.servers()]
             for server in servers:
-                for lines in server.cron(verbose):
-                    yield lines
+                server.cron()
     except lockfile.LockFileBusy:
-        raise SystemExit("ERROR: Another cron is running")
+        output.info("Another cron is running")
 
     except lockfile.LockFilePermissionDenied:
-        raise SystemExit("ERROR: Permission denied, unable to access '%s'"
-                         % filename)
+        output.error("Permission denied, unable to access '%s'",
+                     lockname)
+    output.close_and_exit()
 
 
 # noinspection PyUnusedLocal
@@ -125,6 +126,7 @@ def backup(args):
     """
     Perform a full backup for the given server
     """
+
     servers = get_server_list(args)
     for name in sorted(servers):
         server = servers[name]
@@ -134,6 +136,7 @@ def backup(args):
         immediate_checkpoint = getattr(args, 'immediate_checkpoint',
                                        server.config.immediate_checkpoint)
         server.backup(immediate_checkpoint)
+
     output.close_and_exit()
 
 
@@ -235,25 +238,30 @@ def recover(args):
     if backup is None or backup.status != BackupInfo.DONE:
         raise SystemExit("ERROR: unknown backup '%s' for server '%s'" % (
             args.backup_id, args.server_name))
-        # decode the tablespace relocation rules
+
+    # decode the tablespace relocation rules
     tablespaces = {}
     if args.tablespace:
         for rule in args.tablespace:
             try:
                 tablespaces.update([rule.split(':', 1)])
-            except:
+            except ValueError:
                 raise SystemExit(
                     "ERROR: invalid tablespace relocation rule '%s'\n"
-                    "HINT: the valid syntax for a relocation rule is NAME:LOCATION" % rule)
-                # validate the rules against the tablespace list
-    valid_tablespaces = [tablespace_data[0] for tablespace_data in
+                    "HINT: the valid syntax for a relocation rule is "
+                    "NAME:LOCATION" % rule)
+
+    # validate the rules against the tablespace list
+    valid_tablespaces = [tablespace_data.name for tablespace_data in
                          backup.tablespaces] if backup.tablespaces else []
-    for tablespace in tablespaces:
-        if tablespace not in valid_tablespaces:
+    for item in tablespaces:
+        if item not in valid_tablespaces:
             raise SystemExit("ERROR: invalid tablespace name '%s'\n"
-                             "HINT: please use any of the following tablespaces: %s"
-                             % (tablespace, ', '.join(valid_tablespaces)))
-            # explicitly disallow the rsync remote syntax (common mistake)
+                             "HINT: please use any of the following "
+                             "tablespaces: %s"
+                             % (item, ', '.join(valid_tablespaces)))
+
+    # explicitly disallow the rsync remote syntax (common mistake)
     if ':' in args.destination_directory:
         raise SystemExit(
             "ERROR: the destination directory parameter cannot contain the ':' character\n"
@@ -316,6 +324,14 @@ def check(args):
         server.check()
     output.close_and_exit()
 
+@expects_obj
+def diagnose(args):
+    """
+    Diagnostic command (for support and problems detection purpose)
+    """
+    servers = get_server_list(None)
+    barman.diagnose.exec_diagnose(servers)
+    output.close_and_exit()
 
 @named('show-backup')
 @arg('server_name',
@@ -449,8 +465,10 @@ def global_config(args):
         _logger.warn('unknown log_level in config file: %s', config.log_level)
 
     # configure output
-    if args.format != output.DEFAULT_WRITER:
-        output.set_output_writer(args.format)
+    if args.format != output.DEFAULT_WRITER or args.quiet or args.debug:
+        output.set_output_writer(args.format,
+                                 quiet=args.quiet,
+                                 debug=args.debug)
 
     # Load additional configuration files
     _logger.debug('Loading additional configuration files')
@@ -530,6 +548,7 @@ def main():
                         % ', '.join(barman.config.Config.CONFIG_FILES),
                    default=SUPPRESS)
     p.add_argument('-q', '--quiet', help='be quiet', action='store_true')
+    p.add_argument('-d', '--debug', help='debug output', action='store_true')
     p.add_argument('-f', '--format', help='output format',
                    choices=output.AVAILABLE_WRITERS.keys(),
                    default=output.DEFAULT_WRITER)
@@ -540,6 +559,7 @@ def main():
             show_server,
             status,
             check,
+            diagnose,
             backup,
             list_backup,
             show_backup,
@@ -552,8 +572,8 @@ def main():
     # noinspection PyBroadException
     try:
         p.dispatch(pre_call=global_config, output_file=_output_stream)
-    except Exception:
-        msg = "Unhandled exception. See log file for more details."
+    except Exception, e:
+        msg = "%s\nSee log file for more details." % e
         output.exception(msg)
         output.close_and_exit()
 
