@@ -53,6 +53,24 @@ def parse_boolean(value):
     raise ValueError("Invalid boolean representation (use 'true' or 'false')")
 
 
+def parse_backup_options(value):
+    """
+    Parse a string to a valid backup_options value.
+
+    At the moment only one option is allowed and it must be
+    either "concurrent_backup" or "exclusive_backup"
+
+    :param str value: backup_options value
+    :raises ValueError: if the value is invalid
+    """
+    if value.lower() == "concurrent_backup":
+        return "concurrent_backup"
+    if value.lower() == "exclusive_backup":
+        return "exclusive_backup"
+    raise ValueError("Invalid value "
+                     "(use 'concurrent_backup' or 'exclusive_backup')")
+
+
 class Server(object):
     """
     This class represents a server.
@@ -67,7 +85,7 @@ class Server(object):
         'retention_policy',
         'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
         'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit',
-        'immediate_checkpoint', 'network_compression',
+        'backup_options', 'immediate_checkpoint', 'network_compression',
     ]
 
     BARMAN_KEYS = [
@@ -77,7 +95,7 @@ class Server(object):
         'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
         'configuration_files_directory',
         'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit',
-        'immediate_checkpoint', 'network_compression',
+        'backup_options', 'immediate_checkpoint', 'network_compression',
     ]
 
     DEFAULTS = {
@@ -90,6 +108,7 @@ class Server(object):
         'retention_policy_mode': 'auto',
         'wal_retention_policy': 'main',
         'minimum_redundancy': '0',
+        'backup_options': 'exclusive_backup',
         'immediate_checkpoint': 'false',
         'network_compression': 'false',
     }
@@ -98,6 +117,7 @@ class Server(object):
         'active': parse_boolean,
         'immediate_checkpoint': parse_boolean,
         'network_compression': parse_boolean,
+        'backup_options': parse_backup_options,
     }
 
     def __init__(self, config, name):
@@ -105,16 +125,29 @@ class Server(object):
         self.name = name
         self.barman_home = config.get('barman', 'barman_home')
         for key in Server.KEYS:
-            value = config.get(name, key, self.__dict__)
+            # Get the setting from the [name] section of config file
+            # A literal None value is converted to an empty string
+            value = config.get(name, key, self.__dict__, none_value='')
             source = '[%s] section' % name
+
+            # If the setting isn't present in [name] section of config file
+            # check if it has to be inherited from the [barman] section
             if value is None and key in Server.BARMAN_KEYS:
                 value = config.get('barman', key)
                 source = '[barman] section'
+
+            # If the setting isn't present in [name] section of config file
+            # and is not inherited from global section use its default
+            # (if present)
             if value is None and key in Server.DEFAULTS:
                 value = Server.DEFAULTS[key] % self.__dict__
                 source = 'DEFAULTS'
+            # An empty string is a None value (bypassing inheritance
+            # from global configuration)
+            if value is not None and len(value) == 0:
+                value = None
             # If we have a parser for the current key use it to obtain the
-            # actual value. If an exception is trown output a warning and
+            # actual value. If an exception is thrown output a warning and
             # ignore the value.
             # noinspection PyBroadException
             try:
@@ -124,6 +157,7 @@ class Server(object):
                 output.warning("Invalid configuration value '%s' for key %s"
                                " in %s: %s",
                                value, key, source, e)
+                _logger.exception(e)
             setattr(self, key, value)
 
 
@@ -160,7 +194,7 @@ class Config(object):
         self._servers = None
         self._parse_global_config()
 
-    def get(self, section, option, defaults=None):
+    def get(self, section, option, defaults=None, none_value=None):
         """Method to get the value from a given section from
         Barman configuration
         """
@@ -168,8 +202,8 @@ class Config(object):
             return None
         try:
             value = self._config.get(section, option, raw=False, vars=defaults)
-            if value == 'None':
-                value = None
+            if value.lower() == 'none':
+                value = none_value
             if value is not None:
                 value = self._QUOTE_RE.sub(lambda m: m.group(2), value)
             return value
