@@ -279,15 +279,15 @@ def set_output_writer(new_writer, *args, **kwargs):
 
 
 class ConsoleOutputWriter(object):
-    def __init__(self, debug_enabled=False):
+    def __init__(self, debug=False, quiet=False):
         """
         Default output writer that output everything on console.
 
-        If debug_enabled print the debug information on console.
-
-        :param bool debug_enabled:
+        :param bool debug: print debug messages on standard error
+        :param bool quiet: don't print info messages
         """
-        self._debug = debug_enabled
+        self._debug = debug
+        self._quiet = quiet
 
         #: Used in check command to hold the check results
         self.result_check_list = []
@@ -322,7 +322,8 @@ class ConsoleOutputWriter(object):
         """
         Normal messages are sent to standard output
         """
-        self._out(message, args)
+        if not self._quiet:
+            self._out(message, args)
 
     def warning(self, message, *args):
         """
@@ -443,8 +444,8 @@ class ConsoleOutputWriter(object):
                              pretty_size(backup_size),
                              pretty_size(wal_size)))
             if backup_info.tablespaces:
-                tablespaces = [("%s:%s" % (name, location))
-                               for name, _, location in backup_info.tablespaces]
+                tablespaces = [("%s:%s" % (tablespace.name, tablespace.location))
+                               for tablespace in backup_info.tablespaces]
                 out_list.append(' (tablespaces: %s)' %
                                 ', '.join(tablespaces))
             if retention_status:
@@ -470,8 +471,9 @@ class ConsoleOutputWriter(object):
             self.info("  PGDATA directory  : %s", data['pgdata'])
             if data['tablespaces']:
                 self.info("  Tablespaces:")
-                for name, oid, location in data['tablespaces']:
-                    self.info("    %s: %s (oid: %s)", name, location, oid)
+                for item in data['tablespaces']:
+                    self.info("    %s: %s (oid: %s)",
+                              item.name, item.location, item.oid)
             self.info("")
             self.info("  Base backup information:")
             self.info("    Disk usage      : %s",
@@ -575,7 +577,7 @@ class ConsoleOutputWriter(object):
         :param str server_name: the server we are displaying
         :param dict server_info: a dictionary containing the info to display
         """
-        for status, message in server_info.items():
+        for status, message in sorted(server_info.items()):
             self.info("\t%s: %s", status, message)
 
 
