@@ -15,26 +15,36 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-'''This module represents a Server.
+"""
+This module represents a Server.
 Barman is able to manage multiple servers.
-'''
+"""
 
-from barman import xlog, output
-from barman.infofile import BackupInfo, UnknownBackupIdException
-from barman.lockfile import LockFile
+import os
+import re
+import logging
+import datetime
+from contextlib import contextmanager
+
+import psycopg2
+
+from barman import output
+from barman.infofile import BackupInfo, UnknownBackupIdException, Tablespace
+from barman.lockfile import LockFile, LockFileBusy, \
+    LockFilePermissionDenied
 from barman.backup import BackupManager
 from barman.command_wrappers import Command
-from barman.retention_policies import RetentionPolicyFactory, SimpleWALRetentionPolicy
-import os
-import logging
-import psycopg2
-from contextlib import contextmanager
-import itertools
+from barman.retention_policies import RetentionPolicyFactory
+import xlog
+
 
 _logger = logging.getLogger(__name__)
 
+
 class Server(object):
-    '''This class represents a server to backup'''
+    """
+    This class represents a server to backup
+    """
     XLOG_DB = "xlog.db"
 
     def __init__(self, config):
@@ -57,7 +67,7 @@ class Server(object):
         if self.config.bandwidth_limit:
             try:
                 self.config.bandwidth_limit = int(self.config.bandwidth_limit)
-            except:
+            except ValueError:
                 _logger.warning('Invalid bandwidth_limit "%s" for server "%s" (fallback to "0")'
                                 % (self.config.bandwidth_limit, self.config.name))
                 self.config.bandwidth_limit = None
@@ -71,7 +81,7 @@ class Server(object):
                     value = int(value)
                     if value != self.config.bandwidth_limit:
                         rules[key] = value
-                except:
+                except ValueError:
                     _logger.warning("Invalid tablespace_bandwidth_limit rule '%s'" % (rule,))
             if len(rules) > 0:
                 self.config.tablespace_bandwidth_limit = rules
@@ -98,43 +108,53 @@ class Server(object):
 
         # Set retention policy mode
         if self.config.retention_policy_mode != 'auto':
-            _logger.warning('Unsupported retention_policy_mode "%s" for server "%s" (fallback to "auto")'
-                            % (self.config.retention_policy_mode, self.config.name))
+            _logger.warning(
+                'Unsupported retention_policy_mode "%s" for server "%s" '
+                '(fallback to "auto")' % (
+                    self.config.retention_policy_mode, self.config.name))
             self.config.retention_policy_mode = 'auto'
 
         # If retention_policy is present, enforce them
         if self.config.retention_policy:
             # Check wal_retention_policy
             if self.config.wal_retention_policy != 'main':
-                _logger.warning('Unsupported wal_retention_policy value "%s" for server "%s" (fallback to "main")'
-                                % (self.config.wal_retention_policy, self.config.name))
+                _logger.warning(
+                    'Unsupported wal_retention_policy value "%s" '
+                    'for server "%s" (fallback to "main")' % (
+                        self.config.wal_retention_policy, self.config.name))
                 self.config.wal_retention_policy = 'main'
             # Create retention policy objects
             try:
-                rp = RetentionPolicyFactory.create(self,
-                    'retention_policy', self.config.retention_policy)
+                rp = RetentionPolicyFactory.create(
+                    self, 'retention_policy', self.config.retention_policy)
                 # Reassign the configuration value (we keep it in one place)
                 self.config.retention_policy = rp
                 _logger.debug('Retention policy for server %s: %s' % (
                     self.config.name, self.config.retention_policy))
                 try:
-                    rp = RetentionPolicyFactory.create(self,
-                        'wal_retention_policy', self.config.wal_retention_policy)
+                    rp = RetentionPolicyFactory.create(
+                        self, 'wal_retention_policy',
+                        self.config.wal_retention_policy)
                     # Reassign the configuration value (we keep it in one place)
-                    self.wal_retention_policy = rp
-                    _logger.debug('WAL retention policy for server %s: %s' % (
-                        self.config.name, self.config.wal_retention_policy))
-                except:
-                    _logger.error('Invalid wal_retention_policy setting "%s" for server "%s" (fallback to "main")' % (
-                        self.config.wal_retention_policy, self.config.name))
-                    self.wal_retention_policy = SimpleWALRetentionPolicy (
-                        self.retention_policy, self)
+                    self.config.wal_retention_policy = rp
+                    _logger.debug(
+                        'WAL retention policy for server %s: %s' % (
+                            self.config.name, self.config.wal_retention_policy))
+                except ValueError:
+                    _logger.exception(
+                        'Invalid wal_retention_policy setting "%s" '
+                        'for server "%s" (fallback to "main")' % (
+                            self.config.wal_retention_policy, self.config.name))
+                    rp = RetentionPolicyFactory.create(
+                        self, 'wal_retention_policy', 'main')
+                    self.config.wal_retention_policy = rp
 
                 self.enforce_retention_policies = True
 
-            except:
-                _logger.error('Invalid retention_policy setting "%s" for server "%s"' % (
-                    self.config.retention_policy, self.config.name))
+            except ValueError:
+                _logger.exception(
+                    'Invalid retention_policy setting "%s" for server "%s"' % (
+                        self.config.retention_policy, self.config.name))
 
     def check(self):
         """
@@ -172,6 +192,17 @@ class Server(object):
         else:
             output.result('check', self.config.name, 'PostgreSQL', False)
             return
+
+        if self.config.backup_options == 'concurrent_backup':
+            if remote_status['pgespresso_installed']:
+                output.result('check', self.config.name,
+                        'pgespresso extension', True)
+            else:
+                output.result('check', self.config.name,
+                          'pgespresso extension',
+                          False,
+                          'required for concurrent backups')
+
         if remote_status['archive_mode'] == 'on':
             output.result('check', self.config.name, 'archive_mode', True)
         else:
@@ -199,7 +230,6 @@ class Server(object):
         """
         Checks backup directories and creates them if they do not exist
         """
-        error = None
         try:
             self._make_directories()
         except OSError, e:
@@ -235,6 +265,12 @@ class Server(object):
                           "PostgreSQL version",
                           "FAILED trying to get PostgreSQL version")
             return
+        if remote_status['pgespresso_installed']:
+            output.result('status', self.config.name, 'pgespresso',
+                          'pgespresso extension', "Available")
+        else:
+            output.result('status', self.config.name, 'pgespresso',
+                          'pgespresso extension', "Not available")
         if remote_status['data_directory']:
             output.result('status', self.config.name,
                           "data_directory",
@@ -289,35 +325,98 @@ class Server(object):
         # Executes the backup manager status info method
         self.backup_manager.status()
 
+    def pg_espresso_installed(self):
+        """
+        Returns true if pgexpresso extension is available
+        """
+        try:
+            with self.pg_connect() as conn:
+                # pg_extension is only available from Postgres 9.1+
+                if self.server_version < 90100:
+                    return False
+                cur = conn.cursor()
+                cur.execute("select count(*) from pg_extension "
+                            "where extname = 'pgespresso'")
+                q_result = cur.fetchone()[0]
+                if q_result > 0:
+                    return True
+                else:
+                    return False
+        except psycopg2.Error, e:
+            _logger.debug("Error retrieving pgespresso information: %s", e)
+            return False
+
+    def pg_is_in_recovery(self):
+        """
+        Returns true if PostgreSQL server is in recovery mode
+        """
+        try:
+            with self.pg_connect() as conn:
+                # pg_is_in_recovery is only available from Postgres 9.0+
+                if self.server_version < 90000:
+                    return False
+                cur = conn.cursor()
+                cur.execute("select pg_is_in_recovery()")
+                q_result = cur.fetchone()[0]
+                if q_result:
+                    return True
+                else:
+                    return False
+        except psycopg2.Error, e:
+            _logger.debug("Error calling pg_is_in_recovery() function: %s", e)
+            return None
+
     def get_remote_status(self):
-        '''Get the status of the remote server'''
-        pg_settings = ('archive_mode', 'archive_command', 'data_directory')
-        pg_query_keys = ('server_txt_version', 'current_xlog')
+        """
+        Get the status of the remote server
+
+        :return: result of the server status query
+        """
+
+        pg_settings = (
+            'archive_mode', 'archive_command', 'data_directory')
+        pg_query_keys = (
+            'server_txt_version', 'current_xlog', 'pgespresso_installed')
+
+        # Initialise the result dictionary setting all the values to None
         result = dict.fromkeys(pg_settings + pg_query_keys, None)
         try:
             with self.pg_connect() as conn:
                 for name in pg_settings:
                     result[name] = self.get_pg_setting(name)
+
                 try:
                     cur = conn.cursor()
                     cur.execute("SELECT version()")
                     result['server_txt_version'] = cur.fetchone()[0].split()[1]
-                except:
-                    result['server_txt_version'] = None
+                except psycopg2.Error, e:
+                    _logger.debug(
+                        "Error retrieving PostgreSQL version: %s", e)
+
+
+                result['pgespresso_installed'] = self.pg_espresso_installed()
+
                 try:
-                    cur = conn.cursor()
-                    cur.execute('SELECT pg_xlogfile_name(pg_current_xlog_location())')
-                    result['current_xlog'] = cur.fetchone()[0];
-                except:
-                    result['current_xlog'] = None
+                    if not self.pg_is_in_recovery():
+                        cur = conn.cursor()
+                        cur.execute(
+                            'SELECT pg_xlogfile_name('
+                            'pg_current_xlog_location())')
+                        result['current_xlog'] = cur.fetchone()[0]
+                except psycopg2.Error, e:
+                    _logger.debug("Error retrieving current xlog: %s", e)
+
                 result.update(self.get_pg_configuration_files())
-        except:
-            pass
+        except psycopg2.Error, e:
+            _logger.warn("Error retrieving PostgreSQL status: %s", e)
+
+        # TODO: replace with RemoteUnixCommand
         cmd = Command(self.ssh_command, self.ssh_options)
         result['last_shipped_wal'] = None
         if result['data_directory'] and result['archive_command']:
-            archive_dir = os.path.join(result['data_directory'], 'pg_xlog', 'archive_status')
-            out = str(cmd.getoutput(None, 'ls', '-tr', archive_dir)[0])
+            archive_dir = os.path.join(result['data_directory'],
+                                       'pg_xlog', 'archive_status')
+            out = str(cmd.getoutput('ls', '-tr', archive_dir)[0])
             for line in out.splitlines():
                 if line.endswith('.done'):
                     name = line[:-5]
@@ -340,13 +439,15 @@ class Server(object):
 
     @contextmanager
     def pg_connect(self):
-        '''A generic function to connect to Postgres using Psycopg2'''
+        """
+        A generic function to connect to Postgres using Psycopg2
+        """
         myconn = self.conn == None
         if myconn:
             self.conn = psycopg2.connect(self.config.conninfo)
             self.server_version = self.conn.server_version
             if (self.server_version >= 90000
-                and 'application_name=' not in self.config.conninfo):
+                    and 'application_name=' not in self.config.conninfo):
                 cur = self.conn.cursor()
                 cur.execute('SET application_name TO barman')
                 cur.close()
@@ -358,72 +459,216 @@ class Server(object):
                 self.conn = None
 
     def get_pg_setting(self, name):
-        '''Get a postgres setting with a given name
+        """
+        Get a postgres setting with a given name
 
         :param name: a parameter name
-        '''
+        """
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
                 cur.execute('SHOW "%s"' % name.replace('"', '""'))
                 return cur.fetchone()[0]
-            except:
+            except psycopg2.Error, e:
+                _logger.debug("Error retrieving PostgreSQL setting '%s': %s",
+                              name.replace('"', '""'), e)
                 return None
 
     def get_pg_tablespaces(self):
-        '''Returns a list of tablespaces or None if not present'''
+        """
+        Returns a list of tablespaces or None if not present
+        """
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
                 if self.server_version >= 90200:
-                    cur.execute("SELECT spcname, oid, pg_tablespace_location(oid) AS spclocation FROM pg_tablespace WHERE pg_tablespace_location(oid) != ''")
+                    cur.execute(
+                        "SELECT spcname, oid, "
+                        "pg_tablespace_location(oid) AS spclocation "
+                        "FROM pg_tablespace "
+                        "WHERE pg_tablespace_location(oid) != ''")
                 else:
-                    cur.execute("SELECT spcname, oid, spclocation FROM pg_tablespace WHERE spclocation != ''")
-                return cur.fetchall()
-            except:
+                    cur.execute(
+                        "SELECT spcname, oid, spclocation "
+                        "FROM pg_tablespace WHERE spclocation != ''")
+                # Generate a list of tablespace objects
+                return [Tablespace._make(item) for item in cur.fetchall()]
+            except psycopg2.Error, e:
+                _logger.debug("Error retrieving PostgreSQL tablespaces: %s", e)
                 return None
 
     def get_pg_configuration_files(self):
-        '''Get postgres configuration files or None in case of error'''
+        """
+        Get postgres configuration files or an empty dictionary in case of error
+        """
         if self.configuration_files:
             return self.configuration_files
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                cur.execute("SELECT name, setting FROM pg_settings WHERE name IN ('config_file', 'hba_file', 'ident_file')")
+                cur.execute("SELECT name, setting FROM pg_settings "
+                            "WHERE name IN ("
+                            "'config_file', 'hba_file', 'ident_file')")
                 self.configuration_files = {}
                 for cname, cpath in cur.fetchall():
                     self.configuration_files[cname] = cpath
                 return self.configuration_files
-            except:
-                return None
+            except psycopg2.Error, e:
+                _logger.debug("Error retrieving PostgreSQL configuration files "
+                              "location: %s", e)
+                return {}
 
     def pg_start_backup(self, backup_label, immediate_checkpoint):
         """
         Execute a pg_start_backup
 
+        :param backup_label: label for the backup
+        :param immediate_checkpoint: Boolean for immediate checkpoint execution
+        """
+        with self.pg_connect() as conn:
+            if (self.config.backup_options != "concurrent_backup" and
+                    self.pg_is_in_recovery()):
+                raise Exception(
+                    'Unable to start a backup because of server recovery state')
+            try:
+                cur = conn.cursor()
+                cur.execute(
+                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* '
+                    'FROM pg_start_backup(%s,%s) as xlog_loc',
+                    (backup_label, immediate_checkpoint))
+                return cur.fetchone()
+            except psycopg2.Error, e:
+                msg = "pg_start_backup(): %s" % e
+                _logger.debug(msg)
+                raise Exception(msg)
+
+    def pgespresso_start_backup(self, backup_label, immediate_checkpoint):
+        """
+        Execute a pgespresso_start_backup
+
         :param backup_label: label for the backup
         :param immediate_checkpoint Boolean for immediate checkpoint execution
         """
+        with self.pg_connect() as conn:
+
+            if (self.config.backup_options == "concurrent_backup" and
+                    not self.pg_espresso_installed()):
+                raise Exception(
+                    'pgespresso extension required for concurrent_backup')
+            try:
+                cur = conn.cursor()
+                cur.execute('SELECT pgespresso_start_backup(%s,%s)',
+                            (backup_label, immediate_checkpoint))
+                return cur.fetchone()[0]
+            except psycopg2.Error, e:
+                msg = "pgexpresso_start_backup(): %s" % e
+                _logger.debug(msg)
+                raise Exception(msg)
+
+    def start_backup(self, label, immediate_checkpoint, backup_info):
+        """
+        start backup wrapper
+
+        :param backup_label: label for the backup
+        :param immediate_checkpoint: Boolean for immediate checkpoint execution
+        :param backup_info: backup_info object
+        :return:
+        """
+        if self.config.backup_options != 'concurrent_backup':
+            start_row = self.pg_start_backup(label, immediate_checkpoint)
+            if start_row:
+                start_xlog, start_file_name, start_file_offset = start_row
+                backup_info.set_attribute("status", "STARTED")
+                backup_info.set_attribute("timeline",
+                                          int(start_file_name[0:8], 16))
+                backup_info.set_attribute("begin_xlog", start_xlog)
+                backup_info.set_attribute("begin_wal", start_file_name)
+                backup_info.set_attribute("begin_offset", start_file_offset)
+
+        elif self.config.backup_options == 'concurrent_backup':
+            backup_data = self.pgespresso_start_backup(label,
+                                                       immediate_checkpoint)
+            if backup_data:
+                e = re.compile('^START WAL LOCATION: (.*) \(file (.*)\)',
+                               re.MULTILINE)
+                b_info = e.search(backup_data)
+                backup_info.set_attribute("status", "STARTED")
+                backup_info.set_attribute("timeline",
+                                          int(b_info.group(2)[0:8], 16))
+                backup_info.set_attribute("begin_xlog", b_info.group(1))
+                backup_info.set_attribute("begin_wal", b_info.group(2))
+                backup_info.set_attribute("begin_offset",
+                                          xlog.get_offset_from_location(
+                                              b_info.group(1)))
+                backup_info.set_attribute("backup_label", backup_data)
+
+    def pg_stop_backup(self):
+        """
+        Execute a pg_stop_backup
+        """
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                cur.execute('SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* from pg_start_backup(%s,%s) as xlog_loc',
-                            (backup_label, immediate_checkpoint))
+                cur.execute(
+                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* '
+                    'from pg_stop_backup() as xlog_loc')
                 return cur.fetchone()
-            except:
+            except psycopg2.Error, e:
+                _logger.debug("Error issuing pg_stop_backup() command: %s", e)
                 return None
 
-    def pg_stop_backup(self):
-        '''Execute a pg_stop_backup'''
+    def pgespresso_stop_backup(self, backup_label):
+        """
+        Execute a pgespresso_stop_backup
+        """
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                cur.execute('SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* from pg_stop_backup() as xlog_loc')
-                return cur.fetchone()
-            except:
+                cur.execute('SELECT pgespresso_stop_backup(%s)', (backup_label,))
+                return cur.fetchone()[0]
+            except psycopg2.Error, e:
+                _logger.debug(
+                    "Error issuing pgespresso_stop_backup() command: %s", e)
                 return None
 
+    def stop_backup(self, backup_info):
+        """
+        stop backup wrapper
+
+        :param backup_label: label for the backup
+        :param immediate_checkpoint: Boolean for immediate checkpoint execution
+        :param backup_info: backup_info object
+        :return:
+        """
+        if self.config.backup_options != 'concurrent_backup':
+            stop_string = self.pg_stop_backup()
+            if stop_string:
+                stop_xlog, stop_file_name, stop_file_offset = stop_string
+                backup_info.set_attribute("end_time", datetime.datetime.now())
+                backup_info.set_attribute("end_xlog", stop_xlog)
+                backup_info.set_attribute("end_wal", stop_file_name)
+                backup_info.set_attribute("end_offset", stop_file_offset)
+            else:
+                self.current_action = "starting the backup: PostgreSQL server " \
+                                      "" \
+                                      "is already in exclusive backup mode"
+                raise Exception('concurrent exclusive backups are not allowed')
+
+        elif self.config.backup_options == 'concurrent_backup':
+
+            end_wal = self.pgespresso_stop_backup(backup_info.backup_label)
+            if end_wal:
+                decoded_segment = xlog.decode_segment_name(end_wal)
+                backup_info.set_attribute("end_time", datetime.datetime.now())
+                backup_info.set_attribute("end_xlog",
+                                          "%X/%X" % (decoded_segment[1],
+                                                     (decoded_segment[
+                                                          2] + 1) << 24))
+                backup_info.set_attribute("end_wal", end_wal)
+                backup_info.set_attribute("end_offset", 0)
+            else:
+                raise Exception('Concurrent stop backup failed')
+
     def delete_backup(self, backup):
         '''Deletes a backup
 
@@ -432,15 +677,32 @@ class Server(object):
         return self.backup_manager.delete_backup(backup)
 
     def backup(self, immediate_checkpoint):
-        '''Performs a backup for the server'''
+        """
+        Performs a backup for the server
+
+        :param immediate_checkpoint: Boolean for immediate checkpoint execution
+        """
         try:
             # check required backup directories exist
             self._make_directories()
         except OSError, e:
             output.error('failed to create %s directory: %s',
                          e.filename, e.strerror)
-        else:
-            self.backup_manager.backup(immediate_checkpoint)
+            return
+
+        filename = os.path.join(
+            self.config.barman_home, '.%s-backup.lock' % self.config.name)
+
+        try:
+            # lock acquisition and backup execution
+            with LockFile(filename, raise_if_fail=True):
+                self.backup_manager.backup(immediate_checkpoint)
+
+        except LockFileBusy:
+            output.error("Another backup process is running")
+
+        except LockFilePermissionDenied, e:
+            output.error("Permission denied, unable to access '%s'" % e)
 
     def get_available_backups(self, status_filter=BackupManager.DEFAULT_STATUS_FILTER):
         '''Get a list of available backups
@@ -488,38 +750,47 @@ class Server(object):
             output.result('list_backup', backup, backup_size, wal_size, rstatus)
 
     def get_backup(self, backup_id):
-        '''Return the backup information for the given backup,
+        """
+        Return the backup information for the given backup,
         or None if its status is not empty
 
         :param backup_id: the ID of the backup to return
-        '''
+        """
         try:
             backup = BackupInfo(self, backup_id=backup_id)
             if backup.status in BackupInfo.STATUS_NOT_EMPTY:
                 return backup
             return None
-        except:
+        except Exception, e:
+            _logger.debug("Error reading backup information for %s %s: %s",
+                          self.config.name, backup_id, e, exc_info=1)
             return None
 
     def get_previous_backup(self, backup_id):
-        '''Get the previous backup (if any) from the catalog
+        """
+        Get the previous backup (if any) from the catalog
 
         :param backup_id: the backup id from which return the previous
-        '''
+        """
         return self.backup_manager.get_previous_backup(backup_id)
 
     def get_next_backup(self, backup_id):
-        '''Get the next backup (if any) from the catalog
+        """
+        Get the next backup (if any) from the catalog
 
         :param backup_id: the backup id from which return the next
-        '''
+        """
         return self.backup_manager.get_next_backup(backup_id)
 
-    def get_required_xlog_files(self, backup, target_tli=None, target_time=None, target_xid=None):
-        '''Get the xlog files required for a backup'''
+    def get_required_xlog_files(self, backup, target_tli=None, target_time=None,
+                                target_xid=None):
+        """
+        Get the xlog files required for a backup
+        """
         begin = backup.begin_wal
         end = backup.end_wal
-        # If timeline isn't specified, assume it is the same timeline of the backup
+        # If timeline isn't specified, assume it is the same timeline
+        # of the backup
         if not target_tli:
             target_tli, _, _ = xlog.decode_segment_name(end)
         with self.xlogdb() as fxlogdb:
@@ -541,10 +812,11 @@ class Server(object):
 
     # TODO: merge with the previous
     def get_wal_until_next_backup(self, backup):
-        '''Get the xlog files between backup and the next
+        """
+        Get the xlog files between backup and the next
 
         :param backup: a backup object, the starting point to retrieve wals
-        '''
+        """
         begin = backup.begin_wal
         next_end = None
         if self.get_next_backup(backup.backup_id):
@@ -592,11 +864,18 @@ class Server(object):
         return self.backup_manager.recover(backup, dest, tablespaces, target_tli, target_time, target_xid, target_name, exclusive, remote_command)
 
     def cron(self, verbose=True):
-        '''Maintenance operations
-
-        :param verbose: turn on verbose mode. default True
-        '''
-        return self.backup_manager.cron(verbose)
+        """
+        Maintenance operations
+        """
+        filename = os.path.join(self.config.barman_home,
+                                '.%s-cron.lock' % self.config.name)
+        try:
+            with LockFile(filename, raise_if_fail=True, wait=True):
+                return self.backup_manager.cron(verbose=verbose)
+        except LockFilePermissionDenied, e:
+            output.error("Permission denied, unable to access '%s'" % e)
+        except (OSError, IOError), e:
+            output.error("%s", e)
 
     @contextmanager
     def xlogdb(self, mode='r'):
@@ -649,7 +928,7 @@ class Server(object):
             compression = None
             try:
                 name, size, stamp = line.split()
-            except:
+            except ValueError:
                 raise ValueError("cannot parse line: %r" % (line,))
         return name, int(size), float(stamp), compression
 
